module Structured
  class Dictionary
    include Structured::Type

    attr_reader :member_klass

    def initialize(member_klass)
      @member_klass = member_klass
    end

    def self.[](member_klass)
      new(member_klass)
    end

    def parse(parsed_yaml, stack:, context: nil)
      unless parsed_yaml.is_a?(Hash)
        raise Structured::Errors::InvalidValue.new(stack,
          "#{stack} expects a hash, but found #{parsed_yaml.class.name}")
      end

      result = {}
      errors = []
      parsed_yaml.each do |key, value|
        begin
          unless key.is_a?(::String)
            raise Structured::Errors::InvalidValue.new(stack,
              "Dictionary key #{key.inspect} is not a string")
          end
          result[key] = member_klass.parse(value, stack: stack.push(key, member_klass), context: context)
        rescue Structured::Errors::Base => exception
          errors << exception
        end
      end

      raise Structured::Errors::MultipleErrors.union(stack, errors) unless errors.empty?
      result
    end

    def format_stack_frame_element(key)
      "[#{key}]"
    end

    def default_value
      {}.freeze
    end

    def type_name
      "Dictionary[#{member_klass.type_name}]"
    end
  end
end
