module Structured
  module Errors
    class MultipleErrors < Base
      attr_reader :root, :errors

      def initialize(stack, errors)
        @errors = errors

        messages = errors.map { |msg| "- #{msg}" }.join("\n")
        super(stack, "Failed to parse #{stack.type.type_name} due to #{errors.length} errors:\n#{messages}")
      end

      def annotation(header: true)
        annotations = errors.map { |error| error.annotation(header: false) }
        if header
          head = "Failed to compile #{CGI.escape_html(stack.root.type.type_name)} due to #{errors.length} errors:"
          head + "\n\n" + annotations.join("\n\n")
        else
          annotations.join("\n\n")
        end
      end

      def self.union(stack, exceptions)
        return exceptions.first if exceptions.size == 1

        flattened_exceptions = exceptions.flat_map do |exception|
          case exception
          when MultipleErrors then exception.errors
          when Base then [exception]
          else raise "Cannot wrap a #{exception.class.name} in a ParsingFailed exception"
          end
        end

        new(stack, flattened_exceptions)
      end
    end
  end
end
