module Structured
  class List
    include Structured::Type

    attr_reader :member_klass

    def initialize(member_klass)
      @member_klass = member_klass
    end

    def self.[](member_klass)
      new(member_klass)
    end

    def parse(parsed_yaml, stack:, context: nil)
      unless parsed_yaml.is_a?(Array)
        raise Structured::Errors::InvalidValue.new(stack, "#{stack} expects an array")
      end

      errors = []
      result = parsed_yaml.map.with_index do |value, index|
        begin
          member_klass.parse(value, stack: stack.push(index, member_klass), context: context)
        rescue Structured::Errors::Base => exception
          errors << exception
        end
      end

      raise Structured::Errors::MultipleErrors.union(stack, errors) unless errors.empty?
      result
    end

    def format_stack_frame_element(index)
      "[#{index}]"
    end

    def default_value
      [].freeze
    end

    def type_name
      "List[#{member_klass.type_name}]"
    end
  end
end
