module Structured
  class ParserStack
    attr_accessor :type, :element, :parent

    def initialize(type:, element:, parent:)
      @type = type
      @element = element
      @parent = parent
    end

    def push(element, type)
      self.class.new(type: type, element: element, parent: self)
    end

    def path
      if root?
        type.type_name
      elsif element.nil?
        # abtract elements like Step use no element, they just forward parsing to a concrete type.
        parent.path
      else
        parent.path + parent.type.format_stack_frame_element(element)
      end
    end

    alias_method :to_s, :path

    def root?
      parent.nil?
    end

    def root
      root? ? self : parent.root
    end

    def self.root(type)
      new(type: type, element: nil, parent: nil)
    end
  end
end
